Some drivers might want to execute code for each port at probe time, as
opposed to executing code just-in-time for the port selected for
networking.

To cater to that use case, introduce a .port_probe() callback method
into the DSA switch operations which is called for each available port,
at the end of dsa_port_probe().

Signed-off-by: Vladimir Oltean <vladimir.olt...@nxp.com>
Reviewed-by: Ramon Fried <rfried....@gmail.com>
Tested-by: Michael Walle <mich...@walle.cc>
---
 include/net/dsa.h | 5 ++++-
 net/dsa-uclass.c  | 8 ++++++++
 2 files changed, 12 insertions(+), 1 deletion(-)

diff --git a/net/dsa-uclass.c b/net/dsa-uclass.c
index dbd8558b6486..9b8ae1e82b92 100644
--- a/net/dsa-uclass.c
+++ b/net/dsa-uclass.c
@@ -270,6 +270,7 @@ static void dsa_port_set_hwaddr(struct udevice *pdev, 
struct udevice *master)
 static int dsa_port_probe(struct udevice *pdev)
 {
        struct udevice *dev = dev_get_parent(pdev);
+       struct dsa_ops *ops = dsa_get_ops(dev);
        struct dsa_port_pdata *port_pdata;
        struct dsa_priv *dsa_priv;
        struct udevice *master;
@@ -299,6 +300,13 @@ static int dsa_port_probe(struct udevice *pdev)
 
        dsa_port_set_hwaddr(pdev, master);
 
+       if (ops->port_probe) {
+               err = ops->port_probe(dev, port_pdata->index,
+                                     port_pdata->phy);
+               if (err)
+                       return err;
+       }
+
        return 0;
 }
 
diff --git a/include/net/dsa.h b/include/net/dsa.h
index 0f31a908c9d1..ab2a9dfbea2d 100644
--- a/include/net/dsa.h
+++ b/include/net/dsa.h
@@ -57,7 +57,8 @@
 /**
  * struct dsa_ops - DSA operations
  *
  * @port_enable:  Initialize a switch port for I/O.
  * @port_disable: Disable I/O for a port.
  * @xmit:         Insert the DSA tag for transmission.
  *                DSA drivers receive a copy of the packet with headroom and
@@ -69,6 +70,8 @@
  *                master including any additional headers.
  */
 struct dsa_ops {
+       int (*port_probe)(struct udevice *dev, int port,
+                         struct phy_device *phy);
        int (*port_enable)(struct udevice *dev, int port,
                           struct phy_device *phy);
        void (*port_disable)(struct udevice *dev, int port,
@@ -57,7 +57,8 @@
  * struct dsa_ops - DSA operations
  *
- * @port_enable:  Initialize a switch port for I/O.
+ * @port_probe:   Initialize a switch port.
+ * @port_enable:  Enable I/O for a port.
  * @port_disable: Disable I/O for a port.
  * @xmit:         Insert the DSA tag for transmission.
-- 
2.25.1

Reply via email to